# -*- coding: utf-8 -*-
"""docutils writer to slides in beamer LaTeX package."""

__docformat__ = 'reStructuredText'
__author__ = u"Łukasz Cieśnik <lukasz.ciesnik@gmail.com>"
__version__ = '0.1'

from docutils import nodes, writers
from docutils.core import publish_cmdline, default_description
from docutils.writers.latex2e import Babel
from slides.latex_util import *
from pygments import highlight
from pygments.lexers import get_lexer_by_name
from pygments.formatters import LatexFormatter
from . import parser_ext

__all__ = ['Writer']

Translator_base = nodes.GenericNodeVisitor



#class BeamerTranslator(nodes.SparseNodeVisitor):
class BeamerTranslator(Translator_base):

    document_class = 'beamer'

    def __init__(self, document):
        Translator_base.__init__(self, document)
        self.settings = document.settings
        self.document_options = [] # TODO
        self.body = []
        self.head = []
        self.docinfo = {}
        self.ignore = 0
        self.section_level = 0
        self.in_slide = False

        self.babel = Babel(self.settings.language_code)
        if self.babel.get_language():
            self.document_options.append(self.babel.get_language())

        self.head.extend([
                usepackage('babel'),
                usepackage('hyperref'),
                usepackage('graphicx'), # 'pdftex'),
                usepackage('color'),
                usepackage('fancyvrb'),
                ])
        latex_encoding = self.to_latex_encoding(self.settings.output_encoding)
        if latex_encoding.startswith('utf8'):
            self.head.extend([usepackage('ucs'), usepackage('inputenc', 'utf8x')])
        else:
            self.head.append(usepackage('inputenc', latex_encoding))
        if self.settings.font_encoding:
            self.head.append(
                usepackage('fontenc', self.settings.font_encoding))
        theme = self.settings.theme
        if theme:
            self.head.append(cmd.usetheme(theme))
        if self.settings.fullscreen:
            self.head.append(cmd.hypersetup(LaTeX('pdfpagemode={FullScreen}')))

        self.code_block_formater = LatexFormatter()
        self.head.append(LaTeX(self.code_block_formater.get_style_defs()))

    def astext(self):
        output = [
            cmd.documentclass(self.document_class, self.document_options),
            self.head,
            self.metadata(),
            env.document.begin(),
            self.title_page(),
            self.body,
            env.document.end(),
            ]
        return tolatex(output) + '\n'

    def title_page(self):
        return env.frame([
                #cmd.frametitle(self.docinfo['title']),
                cmd.maketitle(),
                ])

    def metadata(self):
        metadata = []
        #cmd.hypersetup(','.join('pdftitile={%s}'))
        docinfo = self.docinfo
        contact = docinfo.pop('contact', None)
        if contact:
            docinfo.setdefault('author', [])
            docinfo['author'].extend(contact)
        for name, value in sorted(docinfo.items()):
            if name == 'organization':
                name = 'institute'
            if name == 'author':
                value = '\\and '.join(value)
            metadata.append(Command(name)(value))
        return metadata

    def visit_docinfo(self, node):
        self.ignore += 1

    def depart_docinfo(self, node):
        self.ignore -= 1

    def visit_docinfo_item(self, node):
        name = node.__class__.__name__
        if name == 'abstract':
            self.body.append(cmd.abstract(node))
        elif name == 'dedication':
            self.body.append(env.quote(node))
        else:
            self.docinfo.setdefault(name, [])
            self.docinfo[name].append(node.astext())

    def depart_docinfo_item(self, node):
        pass

    def visit_topic(self, node):
        if 'contents' in node['classes']:
            self.append(cmd.tableofcontents(optional=[]))
        self.ignore += 1

    def depart_topic(self, node):
        self.ignore -= 1

    def visit_section(self, node):
        title = node[node.first_child_matching_class(nodes.title)]
        self.last_title = title.astext()
        command = Command(self.section_levels[self.section_level])
        self.body.append(command(title))
        self.section_level += 1

    section_levels = [
        'section',
        'subsection',
        'subsubsection',
        'paragraph',
        'subparagraph',
    ]

    def depart_section(self, node):
        self.section_level -= 1
        self.close_slide()

    def close_slide(self):
        if self.in_slide:
            self.body.append(env.frame.end())
            self.in_slide = False

    def visit_title(self, node):
        self.ignore += 1
        if not self.body:
            self.visit_docinfo_item(node)

    def depart_title(self, node):
        self.ignore -= 1

    def visit_subtitle(self, node):
        self.ignore += 1

    def depart_subtitle(self, node):
        self.ignore -= 1

    #def visit_paragraph(self, node):
    #    self.append('\\par ')

    def depart_paragraph(self, node):
        self.append('\n\n')

    def visit_Text(self, node):
        self.append(node)

    def visit_bullet_list(self, node):
        self.append(env.itemize.begin())

    def depart_bullet_list(self, node):
        self.append(env.itemize.end())

    enum_styles = {'arabic': 'arabic', 'loweralpha': 'alph', 'upperalpha':
                   'Alph', 'lowerroman': 'roman', 'upperroman': 'Roman'}

    def visit_enumerated_list(self, node):
        #enum_prefix = self.encode(node['prefix'])
        #enum_suffix = self.encode(node['suffix'])
        #enum_type = '\\' + self.enum_styles.get(node['enumtype'], r'arabic')
        self.append(env.enumerate.begin())

    def depart_enumerated_list(self, node):
        self.append(env.enumerate.end())

    def visit_list_item(self, node):
        self.append(cmd.item)

    def visit_literal_block(self, node):
        self.append([
                env.verbatim.begin(),
                LaTeX(node.astext()),
                env.verbatim.end(),
                ])
        raise nodes.SkipNode

    def visit_code_block(self, node):
        lexer = get_lexer_by_name(node['syntax'])
        result = highlight(node.astext(), lexer, self.code_block_formater)
        if node.get('inline', False):
            result = result.replace('Verbatim', 'BVerbatim')
        else:
            result = result.replace(r'\begin{Verbatim}[',
                r'\begin{Verbatim}[xleftmargin=1em,')
            #result = env.block(LaTeX(result), mandatory=['Code'])
        self.append(LaTeX(result))
        raise nodes.SkipNode

    def visit_reference(self, node):
        hash_char = '\\#'
        if node.has_key('refuri'):
            href = node['refuri'].replace('#',hash_char)
        elif node.has_key('refid'):
            href = hash_char + node['refid']
        elif node.has_key('refname'):
            href = hash_char + self.document.nameids[node['refname']]
        else:
            raise AssertionError('unknown reference')
        self.append(cmd.href(node.astext(), mandatory=[LaTeX(href)]))
        self.ignore += 1

    def depart_reference(self, node):
        self.ignore -= 1

    def visit_emphasis(self, node):
        self.append(cmd.emph.begin())

    def depart_emphasis(self, node):
        self.append(cmd.emph.end())

    def visit_strong(self, node):
        self.append(cmd.textbf.begin())

    def depart_strong(self, node):
        self.append(cmd.textbf.end())

    def visit_english(self, node):
        self.append(cmd.emph.begin())

    def depart_english(self, node):
        self.append(cmd.emph.end())

    def visit_note(self, node):
        #self.append(LaTeX('\\mode<article>{\n'))
        self.append(LaTeX('\\note{\n'))

    def depart_note(self, node):
        self.append(LaTeX('}\n'))

    def append(self, item):
        if self.ignore:
            return
        if not self.in_slide:
            if hasattr(self, 'last_title'):
                self.body.append(env.frame.begin(['fragile']))
                self.body.append(cmd.frametitle(self.last_title))
                self.in_slide = True
            else:
                pass
                #self.body.append(item)
        self.body.append(item)

    def visit_comment(self, node):
        self.ignore += 1

    def depart_comment(self, node):
        self.ignore -= 1

    def default_visit(self, node):
        pass
        #print '->', node.__class__

    def default_departure(self, node):
        pass
        #print '<-', node.__class__

    def to_latex_encoding(self, docutils_encoding):
        """
        Translate docutils encoding name into latex's.

        Default fallback method is remove "-" and "_" chars from docutils_encoding.

        """
        tr = {  'iso-8859-1': 'latin1',     # west european
                'iso-8859-2': 'latin2',     # east european
                'iso-8859-3': 'latin3',     # esperanto, maltese
                'iso-8859-4': 'latin4',     # north european,scandinavian, baltic
                'iso-8859-5': 'iso88595',   # cyrillic (ISO)
                'iso-8859-9': 'latin5',     # turkish
                'iso-8859-15': 'latin9',    # latin9, update to latin1.
                'mac_cyrillic': 'maccyr',   # cyrillic (on Mac)
                'koi8-r': 'koi8-r',         # cyrillic (Russian)
                'koi8-u': 'koi8-u',         # cyrillic (Ukrainian)
                'windows-1250': 'cp1250',   #
                'windows-1251': 'cp1251',   # cyrillic (on Windows)
                'windows-1252': 'cp1252',   #
                'us-ascii': 'ascii',        # ASCII (US)
                #'iso-8859-6': ''   # arabic
                #'iso-8859-7': ''   # greek
                #'iso-8859-8': ''   # hebrew
                #'iso-8859-10': ''   # latin6, more complete iso-8859-4
             }
        if tr.has_key(docutils_encoding.lower()):
            return tr[docutils_encoding.lower()]
        # convert: latin-1 and utf-8 and similar things
        return docutils_encoding.replace('_', '').replace('-', '').lower()


docinfo_item_names = [
    'author', 'organization', 'contact', 'address', 'version',
    'status', 'date', 'copyright', 'dedication', 'abstract']
for name in docinfo_item_names:
    setattr(BeamerTranslator, 'visit_%s'%name, BeamerTranslator.visit_docinfo_item)
    setattr(BeamerTranslator, 'depart_%s'%name, BeamerTranslator.depart_docinfo_item)
del name



class BeamerWriter(writers.Writer):

    translator_class = BeamerTranslator

    settings_spec = (
        "LaTeX and Beamer Specific Options",
        "Note that this Beamer writer is still EXPERIMENTAL and not "
        "feature-complete.",
        (
         #('Specify a stylesheet file.  The path is used verbatim to include '
         # 'the file.  Overrides --stylesheet-path.',
         # ['--stylesheet'],
         # {'default': '', 'metavar': '<file>',
         #  'overrides': 'stylesheet_path'}),
         #('Specify a stylesheet file, relative to the current working '
         # 'directory.  Overrides --stylesheet.',
         # ['--stylesheet-path'],
         # {'metavar': '<file>', 'overrides': 'stylesheet',
         #  'default': default_stylesheet_path}),
         #('Specify a user stylesheet file.  See --stylesheet.',
         # ['--user-stylesheet'],
         # {'default': '', 'metavar': '<file>',
         #  'overrides': 'user_stylesheet_path'}),
         #('Specify a user stylesheet file.  See --stylesheet-path.',
         # ['--user-stylesheet-path'],
         # {'metavar': '<file>', 'overrides': 'user_stylesheet'}),
         (
            'Specify theme.',
            ['--theme'],
            {'default': 'Warsaw', }
         ),
         ('LaTeX font encoding. '
          'Possible values are "T1", "OT1", "" or some other fontenc option. '
          'The font encoding influences available symbols, e.g. "<<" as one '
          'character. Default is "" which leads to package "ae" (a T1 '
          'emulation using CM fonts).',
          ['--font-encoding'],
          {'default': ''}),
         (
             "Instruct PDF reader to start presentation in fullscreen mode.",
             ['--fullscreen'],
             {
                 'action': 'store_true',
                 'default': False,
             }
         ),
         ),
        )

    settings_defaults = {
        # Many Unicode characters are provided by unicode_map.py, so
        # we can default to latin-1.
        #'output_encoding': 'latin-1',
        'output_encoding_error_handler': 'strict',
        # During development:
        'traceback': 1
        }

    #def __init__(self):
    #    pass

    def translate(self):
        visitor = self.translator_class(self.document)
        self.document.walkabout(visitor)
        self.output = visitor.astext()


def main():
    description = (
        "Generates Beamer-flavoured LaTeX for PDF-based presentations.  "
        + default_description)
    publish_cmdline(writer=BeamerWriter(), description=description)

